<?php

declare(strict_types=1);

namespace apexl\Io\dto\Relationship\Decorators;

use apexl\Io\interfaces\QueryDecorator;
use apexl\Io\interfaces\RelationshipInterface;
use apexl\Vault\interfaces\driver;

final readonly class Identity implements QueryDecorator
{
    public function decorate(RelationshipInterface $relationship, driver $query): driver
    {
        return $query;
    }
}
