<?php

declare(strict_types=1);

namespace {{namespace}}\Service;

use {{namespace}}\Interface\MyInterface;

class MyService implements MyInterface
{
    public function handle(): void
    {
        /** @TODO Implement handle() method */
    }
}
