<?php

declare(strict_types=1);

namespace apexl\Io\interfaces;

use apexl\Vault\enums\SortDirection;

interface RelationOrderBy
{
    /**
     * @return array<int, array{0: string, 1: SortDirection}>
     */
    public function clauses(RelationshipInterface $relationship): array;
}
